@@ -236,14 +236,9 @@ struct queue *get_a2dp_tests(void)
 	uint16_t i = 0;
 
 	list = queue_new();
-	if (!list)
-		return NULL;
 
 	for (; i < sizeof(test_cases) / sizeof(test_cases[0]); ++i)
-		if (!queue_push_tail(list, &test_cases[i])) {
-			queue_destroy(list, NULL);
-			return NULL;
-		}
+		queue_push_tail(list, &test_cases[i]);
 
 	return list;
 }
