@@ -1105,6 +1105,8 @@ static void connection_lost(struct avdtp *session, int err)
 {
 	char address[18];
 
+	session = avdtp_ref(session);
+
 	ba2str(device_get_address(session->device), address);
 	DBG("Disconnected from %s", address);
 
@@ -1115,10 +1117,7 @@ static void connection_lost(struct avdtp *session, int err)
 
 	avdtp_set_state(session, AVDTP_SESSION_STATE_DISCONNECTED);
 
-	if (session->ref > 0)
-		return;
-
-	avdtp_free(session);
+	avdtp_unref(session);
 }
 
 static gboolean disconnect_timeout(gpointer user_data)
@@ -1171,7 +1170,18 @@ void avdtp_unref(struct avdtp *session)
 	if (session->ref > 0)
 		return;
 
-	set_disconnect_timer(session);
+	switch (session->state) {
+	case AVDTP_SESSION_STATE_CONNECTED:
+		set_disconnect_timer(session);
+		break;
+	case AVDTP_SESSION_STATE_CONNECTING:
+		connection_lost(session, ECONNABORTED);
+		break;
+	case AVDTP_SESSION_STATE_DISCONNECTED:
+	default:
+		avdtp_free(session);
+		break;
+	}
 }
 
 struct avdtp *avdtp_ref(struct avdtp *session)
@@ -3476,7 +3486,7 @@ int avdtp_abort(struct avdtp *session, struct avdtp_stream *stream)
 	if (!stream && session->discover) {
 		/* Don't call cb since it being aborted */
 		session->discover->cb = NULL;
-		finalize_discovery(session, -ECANCELED);
+		finalize_discovery(session, ECANCELED);
 		return -EALREADY;
 	}
 
